@@ -45,7 +45,9 @@ class Twarc(object):
                  access_token=None, access_token_secret=None,
                  connection_errors=0, http_errors=0, config=None,
                  profile="", protected=False, tweet_mode="extended",
-                 app_auth=False, validate_keys=True):
+                 app_auth=False, validate_keys=True, gnip_auth=False,
+                 gnip_username=None, gnip_password=None,
+                 gnip_account=None):
         """
         Instantiate a Twarc instance. If keys aren't set we'll try to
         discover them in the environment or a supplied profile. If no
@@ -53,6 +55,7 @@ class Twarc(object):
         be used.
         """
 
+        self.api_version = "1.1"
         self.consumer_key = consumer_key
         self.consumer_secret = consumer_secret
         self.access_token = access_token
@@ -65,6 +68,10 @@ class Twarc(object):
         self.tweet_mode = tweet_mode
         self.protected = protected
         self.app_auth = app_auth
+        self.gnip_auth = gnip_auth
+        self.gnip_username = gnip_username
+        self.gnip_password = gnip_password
+        self.gnip_account = gnip_account
 
         if config:
             self.config = config
@@ -146,7 +153,7 @@ class Twarc(object):
             to_date=None, max_results=None, sandbox=False, limit=0):
         """
         Search using the Premium Search API. You will need to pass in a query
-        a product (30day or fullarchive) and and environment to use. Optionally
+        a product (30day or fullarchive) and environment to use. Optionally
         you can pass in a from_date and to_date to limit the search using
         datetime objects. If you would like to set max_results you can, or
         you can accept the maximum results (500). If using the a sandbox 
@@ -155,7 +162,7 @@ class Twarc(object):
         return more than that number of tweets (0 means no limit).
         """
 
-        if not self.app_auth:
+        if not self.app_auth and not self.gnip_auth:
             raise RuntimeError(
                 "This endpoint is only available with application authentication. "
                 "Pass app_auth=True in Python or --app-auth on the command line."
@@ -166,7 +173,7 @@ class Twarc(object):
         if to_date and not isinstance(to_date, datetime.date):
             raise RuntimeError("to_date must be a datetime.date or datetime.datetime object")
 
-        if product not in ['30day', 'fullarchive']:
+        if product not in ['30day', 'gnip_fullarchive', 'fullarchive']:
             raise RuntimeError(
                 'Invalid Premium Search API product: {}'.format(product)
             )
@@ -178,10 +185,16 @@ class Twarc(object):
             else:
                 max_results = 500
 
-        url = 'https://api.twitter.com/1.1/tweets/search/{}/{}.json'.format(
-            product, 
-            environment
-        )
+        if product == 'gnip_fullarchive':
+            url = 'https://gnip-api.twitter.com/search/fullarchive/accounts/{}/{}.json'.format(
+                self.gnip_account,
+                environment
+            )
+        else:
+            url = 'https://api.twitter.com/1.1/tweets/search/{}/{}.json'.format(
+                product, 
+                environment
+            )
 
         params = {
             "query": q,
@@ -206,6 +219,8 @@ class Twarc(object):
                     params['next'] = data['next']
                 else:
                     stop = True
+            elif resp.status_code == 422:
+                raise RuntimeError("Twitter API 422 response: are you using a premium search sandbox environment and forgot the --sandbox argument?")
 
     def timeline(self, user_id=None, screen_name=None, max_id=None,
                  since_id=None, max_pages=None):
@@ -527,79 +542,6 @@ class Twarc(object):
                     log.info("stopping filter")
                     return
 
-    def labs_v1_sample(self, event=None, record_keepalive=False):
-        """
-        Returns a small random sample of all public statuses, using the new Twitter
-        labs API version of the endpoint.
-
-        The Tweets returned by the default access level are the same, so if two
-        different clients connect to this endpoint, they will see the same Tweets.
-
-        If a threading.Event is provided for event and the event is set,
-        the sample will be interrupted.
-
-        Requires the use of application only authorisation (set app_auth=True on
-        constructing the Twarc client, or on the commandline).
-        """
-
-        if not self.app_auth:
-            raise RuntimeError(
-                "This endpoint is only available with application authentication. "
-                "Pass app_auth=True in Python or --app-auth on the command line."
-            )
-
-        url = 'https://api.twitter.com/labs/1/tweets/stream/sample'
-        headers = {'accept-encoding': 'deflate, gzip'}
-        errors = 0
-        while True:
-            try:
-                log.info("connecting to labs sample stream")
-                resp = self.get(url, headers=headers, stream=True)
-                errors = 0
-                for raw_line in resp.iter_lines(chunk_size=512):
-                    line = raw_line.decode()
-                    if event and event.is_set():
-                        log.info("stopping sample")
-                        # Explicitly close response
-                        resp.close()
-                        return
-                    if line == "":
-                        log.info("keep-alive")
-                        if record_keepalive:
-                            yield "keep-alive"
-                        continue
-                    try:
-                        yield json.loads(line)
-                    except Exception as e:
-                        log.error("json parse error: %s - %s", e, line)
-            except requests.exceptions.HTTPError as e:
-                errors += 1
-                log.error("caught http error %s on %s try", e, errors)
-                if self.http_errors and errors == self.http_errors:
-                    log.warning("too many errors")
-                    raise e
-                if e.response_status_code == 403:
-                    log.warning("access denied for app (403 Error)")
-                    raise e
-                if e.response.status_code == 420:
-                    if interruptible_sleep(errors * 60, event):
-                        log.info("stopping filter")
-                        return
-                else:
-                    if interruptible_sleep(errors * 5, event):
-                        log.info("stopping filter")
-                        return
-
-            except Exception as e:
-                errors += 1
-                log.error("caught exception %s on %s try", e, errors)
-                if self.http_errors and errors == self.http_errors:
-                    log.warning("too many errors")
-                    raise e
-                if interruptible_sleep(errors, event):
-                    log.info("stopping filter")
-                    return
-
     def dehydrate(self, iterator):
         """
         Pass in an iterator of tweets' JSON and get back an iterator of the
@@ -835,17 +777,12 @@ class Twarc(object):
         if not self.client:
             self.connect()
 
-        # set default tweet_mode
-        if "params" not in kwargs:
-            kwargs["params"] = {"tweet_mode": self.tweet_mode}
-        else:
-            kwargs["params"]["tweet_mode"] = self.tweet_mode
-
-        # override tweet_mode for labs and premium endpoints
-        if re.search(r"api.twitter.com/labs", args[0]):
-            kwargs["params"] = {"format": "detailed"}
-        elif re.search(r"api.twitter.com/1.1/tweets/search/", args[0]):
-            kwargs["params"].pop("tweet_mode")
+        # set default tweet_mode; only used for non-premium/non-gnip endpoints
+        if self.is_standard_v1(args[0]):
+            if "params" not in kwargs:
+                kwargs["params"] = {"tweet_mode": self.tweet_mode}
+            else:
+                kwargs["params"]["tweet_mode"] = self.tweet_mode
 
         # Pass allow 404 to not retry on 404
         allow_404 = kwargs.pop('allow_404', False)
@@ -911,7 +848,9 @@ class Twarc(object):
         Sets up the HTTP session to talk to Twitter. If one is active it is
         closed and another one is opened.
         """
-        if not (self.consumer_key and self.consumer_secret and self.access_token
+        if self.gnip_auth and not (self.gnip_username and self.gnip_password and self.gnip_account):
+            raise RuntimeError("MissingKeys")
+        elif not self.gnip_auth and not (self.consumer_key and self.consumer_secret and self.access_token
                 and self.access_token_secret):
             raise RuntimeError("MissingKeys")
 
@@ -923,7 +862,12 @@ class Twarc(object):
             self.last_response.close()
         log.info("creating http session")
 
-        if not self.app_auth:
+        if self.gnip_auth:
+            logging.info('creating basic user authentication for gnip')
+            s = requests.Session()
+            s.auth = (self.gnip_username, self.gnip_password)
+            self.client = s
+        elif not self.app_auth:
             logging.info('creating OAuth1 user authentication')
             self.client = OAuth1Session(
                 client_key=self.consumer_key,
@@ -957,25 +901,40 @@ class Twarc(object):
             self.access_token = env('ACCESS_TOKEN')
         if not self.access_token_secret:
             self.access_token_secret = env('ACCESS_TOKEN_SECRET')
+        if not self.gnip_username:
+            self.gnip_username = env('GNIP_USERNAME')
+        if not self.gnip_password:
+            self.gnip_password = env('GNIP_PASSWORD')
+        if not self.gnip_account:
+            self.gnip_account = env('GNIP_ACCOUNT')
 
-        if self.config and not (self.consumer_key and
+        if self.config:
+            if self.gnip_auth and not (self.gnip_username and
+                                self.gnip_password and
+                                self.gnip_account):
+                self.load_config()
+            elif not self.gnip_auth and not (self.consumer_key and
                                 self.consumer_secret and
                                 self.access_token and
                                 self.access_token_secret):
-            self.load_config()
+                self.load_config()
 
     def validate_keys(self):
         """
         Validate the keys provided are authentic credentials.
         """
-        url = 'https://api.twitter.com/1.1/account/verify_credentials.json'
+        if self.gnip_auth:
+            url = 'https://gnip-api.twitter.com/metrics/usage/accounts/{}.json'.format(self.gnip_account)
 
-        keys_present = self.consumer_key and self.consumer_secret and \
-                       self.access_token and self.access_token_secret
-
-        if self.app_auth:
+            keys_present = self.gnip_account and self.gnip_username and self.gnip_password
+        elif self.app_auth:
             # no need to validate keys when using OAuth2 App Auth.
             return True
+        else:
+            url = 'https://api.twitter.com/1.1/account/verify_credentials.json'
+
+            keys_present = self.consumer_key and self.consumer_secret and \
+                        self.access_token and self.access_token_secret
 
         if keys_present:
             try:
@@ -1009,8 +968,8 @@ class Twarc(object):
             profile = config.sections()[0]
 
         data = {}
-        for key in ['access_token', 'access_token_secret',
-                    'consumer_key', 'consumer_secret']:
+        keys = ['gnip_username', 'gnip_password', 'gnip_account'] if self.gnip_auth else ['access_token', 'access_token_secret', 'consumer_key', 'consumer_secret']
+        for key in keys:
             try:
                 setattr(self, key, config.get(profile, key))
             except configparser.NoSectionError:
@@ -1030,11 +989,16 @@ class Twarc(object):
             config.remove_section(profile)
 
         config.add_section(profile)
-        config.set(profile, 'consumer_key', self.consumer_key)
-        config.set(profile, 'consumer_secret', self.consumer_secret)
-        config.set(profile, 'access_token', self.access_token)
-        config.set(profile, 'access_token_secret',
-                   self.access_token_secret)
+        if self.gnip_auth:
+            config.set(profile, 'gnip_username', self.access_token_secret)
+            config.set(profile, 'gnip_password', self.access_token_secret)
+            config.set(profile, 'gnip_account', self.access_token_secret)
+        else:
+            config.set(profile, 'consumer_key', self.consumer_key)
+            config.set(profile, 'consumer_secret', self.consumer_secret)
+            config.set(profile, 'access_token', self.access_token)
+            config.set(profile, 'access_token_secret',
+                    self.access_token_secret)
         with open(self.config, 'w') as config_file:
             config.write(config_file)
 
@@ -1057,42 +1021,58 @@ class Twarc(object):
             self.consumer_key = get_input('consumer key: ')
             self.consumer_secret = get_input('consumer secret: ')
 
-        request_token_url = 'https://api.twitter.com/oauth/request_token'
-        oauth = OAuth1(self.consumer_key, client_secret=self.consumer_secret)
-        r = requests.post(url=request_token_url, auth=oauth)
+        answered = False
+        while not answered:
+            print("\nHow would you like twarc to obtain your user keys?\n\n1) generate access keys by visiting Twitter\n2) manually enter your access token and secret\n")
+            answer = get_input('Please enter your choice [1/2] ')
+            if answer == "1":
+                answered = True
+                generate = True
+            elif answer == "2":
+                answered = True
+                generate = False
 
-        credentials = parse_qs(r.text)
-        if not credentials:
-            print("\nError: invalid credentials.")
-            print("Please check that you are copying and pasting correctly and try again.\n")
-            return
+        if generate:
+            request_token_url = 'https://api.twitter.com/oauth/request_token'
+            oauth = OAuth1(self.consumer_key, client_secret=self.consumer_secret)
+            r = requests.post(url=request_token_url, auth=oauth)
 
-        resource_owner_key = credentials.get('oauth_token')[0]
-        resource_owner_secret = credentials.get('oauth_token_secret')[0]
+            credentials = parse_qs(r.text)
+            if not credentials:
+                print("\nError: invalid credentials.")
+                print("Please check that you are copying and pasting correctly and try again.\n")
+                return
 
-        base_authorization_url = 'https://api.twitter.com/oauth/authorize'
-        authorize_url = base_authorization_url + '?oauth_token=' + resource_owner_key
-        print('\nPlease log into Twitter and visit this URL in your browser:\n%s' % authorize_url)
-        verifier = get_input('\nAfter you have authorized the application please enter the displayed PIN: ')
+            resource_owner_key = credentials.get('oauth_token')[0]
+            resource_owner_secret = credentials.get('oauth_token_secret')[0]
 
-        access_token_url = 'https://api.twitter.com/oauth/access_token'
-        oauth = OAuth1(self.consumer_key,
-                       client_secret=self.consumer_secret,
-                       resource_owner_key=resource_owner_key,
-                       resource_owner_secret=resource_owner_secret,
-                       verifier=verifier)
-        r = requests.post(url=access_token_url, auth=oauth)
-        credentials = parse_qs(r.text)
+            base_authorization_url = 'https://api.twitter.com/oauth/authorize'
+            authorize_url = base_authorization_url + '?oauth_token=' + resource_owner_key
+            print('\nPlease log into Twitter and visit this URL in your browser:\n%s' % authorize_url)
+            verifier = get_input('\nAfter you have authorized the application please enter the displayed PIN: ')
 
-        if not credentials:
-            print('\nError: invalid PIN')
-            print('Please check that you entered the PIN correctly and try again.\n')
-            return
+            access_token_url = 'https://api.twitter.com/oauth/access_token'
+            oauth = OAuth1(self.consumer_key,
+                           client_secret=self.consumer_secret,
+                           resource_owner_key=resource_owner_key,
+                           resource_owner_secret=resource_owner_secret,
+                           verifier=verifier)
+            r = requests.post(url=access_token_url, auth=oauth)
+            credentials = parse_qs(r.text)
 
-        self.access_token = resource_owner_key = credentials.get('oauth_token')[0]
-        self.access_token_secret = credentials.get('oauth_token_secret')[0]
+            if not credentials:
+                print('\nError: invalid PIN')
+                print('Please check that you entered the PIN correctly and try again.\n')
+                return
 
-        screen_name = credentials.get('screen_name')[0]
+            self.access_token = resource_owner_key = credentials.get('oauth_token')[0]
+            self.access_token_secret = credentials.get('oauth_token_secret')[0]
+
+            screen_name = credentials.get('screen_name')[0]
+        else:
+            self.access_token = get_input("Enter your Access Token: ")
+            self.access_token_secret = get_input("Enter your Access Token Secret: ")
+            screen_name = "default"
 
         config = self.save_config(screen_name)
         print('\nThe credentials for %s have been saved to your configuration file at %s' % (screen_name, self.config))
@@ -1104,4 +1084,14 @@ class Twarc(object):
     def default_config(self):
         return os.path.join(os.path.expanduser("~"), ".twarc")
 
+    def is_standard_v1(self, url):
+        result = True
+        if url.startswith('https://gnip-api.twitter.com'):
+            result = False
+        elif url.startswith('https://api.twitter.com/1.1/tweets/search/30day'):
+            result = False
+        elif url.startswith('https://api.twitter.com/1.1/tweets/search/fullarchive'):
+            result = False
+        return result
+
 
