@@ -9,6 +9,7 @@ import twarc
 import click
 import logging
 import pathlib
+import datetime
 import requests
 import configobj
 import threading
@@ -16,7 +17,7 @@ import threading
 from click_plugins import with_plugins
 from pkg_resources import iter_entry_points
 
-from twarc import __version__
+from twarc.version import version
 from twarc.handshake import handshake
 from twarc.decorators import cli_api_error
 from twarc.expansions import flatten as flat
@@ -108,7 +109,7 @@ def configure(ctx):
 
     config_dir = pathlib.Path(click.get_app_dir('twarc'))
     if not config_dir.is_dir():
-        config_dir.mkdir()
+        config_dir.mkdir(parents=True)
     config_file = config_dir / 'config'
 
     config = configobj.ConfigObj(unrepr=True)
@@ -139,7 +140,7 @@ def get_version():
     """
     Return the version of twarc that is installed.
     """
-    click.echo(f'twarc v{__version__}')
+    click.echo(f'twarc v{version}')
 
 
 @twarc2.command('search')
@@ -167,7 +168,18 @@ def search(T, query, outfile, since_id, until_id, start_time, end_time, limit, a
     Search for tweets.
     """
     count = 0
-    search_method = T.search_all if archive else T.search_recent
+
+    if archive:
+        search_method = T.search_all
+        # if the user is searching the historical archive the assumption is that
+        # they want to search everything, and not just the previous month which
+        # is the default: https://github.com/DocNow/twarc/issues/434
+        if start_time == None:
+            start_time = datetime.datetime(2006, 3, 21, 0, 0, 0, 0,
+                datetime.timezone.utc)
+    else:
+        search_method = T.search_recent
+
 
     for result in search_method(query, since_id, until_id, start_time, end_time):
         _write(result, outfile, flatten)
@@ -332,7 +344,7 @@ def mentions(T, user_id, outfile, since_id, until_id, start_time, end_time, flat
 @cli_api_error
 def timeline(T, user_id, outfile, since_id, until_id, start_time, end_time, flatten):
     """
-    Retrieve the 800 most recent tweets mentioning the given user.
+    Retrieve the 3200 most recent tweets for the given user.
     """
     for result in T.timeline(user_id, since_id, until_id, start_time, end_time):
         _write(result, outfile, flatten)
@@ -346,6 +358,10 @@ def flatten(infile, outfile):
     """
     "Flatten" tweets, or move expansions inline with tweet objects.
     """
+    if (infile.name == outfile.name):
+        click.echo(click.style(f"💔 Cannot flatten files in-place, specify a different output file!", fg='red'), err=True)
+        return
+
     for line in infile:
         result = json.loads(line)
         _write(result, outfile, True)
@@ -522,3 +538,5 @@ def _write(results, outfile, flatten, pretty=False):
                 click.echo(json.dumps(r, indent=indent), file=outfile)
         else:
             click.echo(json.dumps(results, indent=indent), file=outfile)
+    else:
+        click.echo(json.dumps(results, indent=indent), file=outfile)
